import 'dart:collection';

import 'package:objd/basic/condition.dart';
import 'package:objd/basic/entity.dart';
import 'package:objd/basic/for_list.dart';
import 'package:objd/basic/score.dart';
import 'package:objd/basic/scoreboard.dart';
import 'package:objd/basic/widget.dart';
import 'package:objd/wrappers/execute.dart';

enum DetectType {
	jump
}

class Detect {
	static final HashMap<DetectType, Scoreboard> _boards = HashMap();
	static final HashMap<Entity, Score> _scores = HashMap();

	static Widget load() {
		return For.of([
			..._boards.values,
			..._scores.values.map((v) => v.set(0))
		]);
	}

	static Widget tick() {
		const List<Widget> actions = [];

		_scores.forEach((k, v) {
			actions.add(Execute(as: k, If: Condition.score(v.matchesRange(Range(from: 1))), children: [v.set(0)]));
		});

		return For.of([
			...actions
		]);
	}

	static void _addDetect(Entity entity, DetectType event) {
		final String scoreboardName = 'ct_${event.toString()}';
		final String eventType = 'minecraft.custom:minecraft.${event}';

		if (!_boards.containsKey(event)) {
			_boards[event] = Scoreboard(scoreboardName, type: eventType, addIntoLoad: false);
		}

		if (!_scores.containsKey(entity)) {
			_scores[entity] = Score(entity, scoreboardName, addNew: false);
		}
	}

	static Condition triggered(DetectType event, [Entity entity]) {
		final _entity = entity ?? Entity.Player();
		_addDetect(_entity, event);
		return Condition.score(_scores[_entity].matchesRange(Range(from: 1)));
	}
}
